/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor;

import java.net.URL;
import java.util.HashMap;
import java.util.Locale;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.resource.FontRegistry;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.eclipse.ui.views.properties.PropertySheetPage;
import org.osgi.framework.BundleContext;

import eu.etaxonomy.cdm.api.application.CdmApplicationController;
import eu.etaxonomy.cdm.database.DataSourceNotFoundException;
import eu.etaxonomy.cdm.database.DbSchemaValidation;
import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.cdm.model.common.DefaultTermInitializer;
import eu.etaxonomy.cdm.model.common.init.TermNotFoundException;
import eu.etaxonomy.taxeditor.datasource.CdmDataSourceRepository;
import eu.etaxonomy.taxeditor.datasource.CdmTransactionController;
import eu.etaxonomy.taxeditor.model.CdmSessionDataRepository;

/**
 * The class controlling the plug-in life cycle.
 * </p>
 * <ul>
 * 	<li>Initializes datastore as necessary.</li>
 * 	<li>Initializes CDM application controller.</li>
 * 	<li>Initializes default preferences.</li>
 * 	<li>Stores registries for colors, fonts, images.</li>
 * </ul>
 * 
 * @author p.ciardelli
 * @created 15.05.2008
 * @version 1.0
 */
public class TaxEditorPlugin extends AbstractUIPlugin {
	private static final Logger logger = Logger
			.getLogger(TaxEditorPlugin.class);

	/**
	 * The plug-in ID
	 */
	public static final String PLUGIN_ID = "eu.etaxonomy.taxeditor.plugin";
	/**
	 * The shared instance
	 */
	private static TaxEditorPlugin plugin;

//	private DbSchemaValidation dbSchemaValidation = DbSchemaValidation.VALIDATE;
	private DbSchemaValidation dbSchemaValidation = DbSchemaValidation.UPDATE;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		
		super.start(context);
		plugin = this;
				
		// Check whether this is the first time the application has ever been run
		checkInitialExecution();
		
		// Initialize application controller
		initApplicationController();
		
		// Start a transaction
		CdmTransactionController.startTransaction();
		
		// Forgot what this is ...
		Locale locale = new Locale("en", "", "icbn");
		Locale.setDefault(locale);
	}

	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		CdmTransactionController.commitTransaction();
		plugin = null;
		super.stop(context);
		
		disposeColors();
	}
	
	/**
	 * Returns the shared instance
	 * 
	 * @return the shared instance
	 */
	public static TaxEditorPlugin getDefault() {
		return plugin;
	}

	private void checkInitialExecution() {
		
		// Check in the prefs store whether the application (i.e. its default
		//	data source) has been initialized
		boolean initialized = false;
		if (getPreferenceStore().contains(ITaxEditorConstants.INITIALIZED)) {
			initialized = getPreferenceStore().getBoolean(
					ITaxEditorConstants.INITIALIZED);
		}
	
		// If not ...
		if (!initialized) {
			
			// ... set validation to CREATE
			logger.warn("Initializing datastore");
			dbSchemaValidation = DbSchemaValidation.CREATE;
			
			// ... and note in prefs that application has been initialized
			getPreferenceStore().setValue(ITaxEditorConstants.INITIALIZED, true);
		}
	}
	
	/***************************************************************************
	 * CDM SERVICES
	 **************************************************************************/

	/**
	 * All CDM services are called via the application controller
	 */
	private CdmApplicationController applicationController;

	private CdmApplicationController initApplicationController() {
		
		// Get the "current" - i.e. most recently used - data source
		ICdmDataSource cdmDatasource = CdmDataSourceRepository.getDefault().
				getCurrentDataSource();
		
		// Initialize the controller with the data source
		try {
			applicationController = CdmApplicationController.NewInstance(cdmDatasource, dbSchemaValidation);
		} catch (DataSourceNotFoundException e) {
			// TODO user-friendly failure here
			e.printStackTrace();
		} catch (TermNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		// Set application controller for objects that use it 
		CdmTransactionController.setApplicationController(applicationController);
		CdmDataSourceRepository.getDefault().setCdmApplicationController(applicationController);
		CdmSessionDataRepository.getDefault().setApplicationController(applicationController);
		
//		// Terms have to be initialized explicitly as of now
//		logger.warn("MAC Workaround enabled!!");
//		DefaultTermInitializer vocabularyStore = new DefaultTermInitializer();
//		vocabularyStore.initialize();
		
		return applicationController;
	}
	
	public CdmApplicationController getApplicationController() {
		if (applicationController == null) {
			throw new IllegalStateException("CdmApplicationController not yet set.");
		}
		return applicationController;
	}

	/***************************************************************************
	 * IMAGE REGISTRY
	 **************************************************************************/
	public ImageDescriptor getImageDescriptor(String key) {
		return getImageRegistry().getDescriptor(key);
	}

	public Image getImage(String key) {
		return getImageRegistry().get(key);
	}

	protected void initializeImageRegistry(ImageRegistry registry) {
		registerImage(registry, ITaxEditorConstants.EDIT_ICON, "edit_16x16.ico");
		registerImage(registry, ITaxEditorConstants.WARNING_ICON,
				"warn_tsk.gif");
		registerImage(registry, ITaxEditorConstants.BLACK_SQUARE_ICON,
				"accepted_small.gif");
		registerImage(registry, ITaxEditorConstants.HOMOTYPIC_SYN_ICON,
				"homosyn_no_bg.gif");
		registerImage(registry,
				ITaxEditorConstants.HOMOTYPIC_SYN_ORIGINAL_ICON,
				"homosyn_original_no_bg.gif");
		registerImage(registry, ITaxEditorConstants.HETEROTYPIC_SYN_ICON,
				"heterosyn_no_bg.gif");
		registerImage(registry,
				ITaxEditorConstants.HETEROTYPIC_SYN_ORIGINAL_ICON,
				"heterosyn_original_no_bg.gif");
		registerImage(registry, ITaxEditorConstants.MISAPPLIED_NAME_ICON,
				"misapplied_no_bg.gif");
		registerImage(registry, ITaxEditorConstants.CONCEPT_ICON,
				"concept_no_bg.gif");
		registerImage(registry, ITaxEditorConstants.AUTONYM_ICON,
				"autonym_no_bg.gif");
		registerImage(registry, ITaxEditorConstants.BASIONYM_ICON,
				"basionym_no_bg.gif");
		registerImage(registry, ITaxEditorConstants.ORTHOGRAPHIC_VARIANT_ICON,
				"orthovariant_no_bg.gif");
		registerImage(registry, ITaxEditorConstants.DB_ICON, "db.gif");
		registerImage(registry, ITaxEditorConstants.MOVE_ICON,
				"correction_change.gif");
		registerImage(registry, ITaxEditorConstants.ACTIVE_DELETE_ICON,
				"delete_edit.gif");
		registerImage(registry, ITaxEditorConstants.SYNONYM_TO_TAXON_ICON,
				"change.gif");
		registerImage(registry, ITaxEditorConstants.OPEN_TAXON_ICON, "open.gif");
		registerImage(registry, ITaxEditorConstants.ADD_CHILD_TAXON_ICON,
				"new_child.gif");
		registerImage(registry,
				ITaxEditorConstants.SWAP_SYNONYM_AND_TAXON_ICON, "swap2.gif");
		registerImage(registry, ITaxEditorConstants.QUICK_ADD_ICON,
				"quick_add.gif");
		registerImage(registry, ITaxEditorConstants.TAXON_TO_SYNONYM_ICON,
				"tax_to_syn.gif");
		registerImage(registry, ITaxEditorConstants.ERROR_ANNOTATION_ICON,
				"error_co.gif");
		registerImage(registry, ITaxEditorConstants.EDIT_BITMAP_ICON,
				"256color_16x16.bmp");
	}

	private void registerImage(ImageRegistry registry, String key,
			String fileName) {
		try {
			IPath path = new Path("icons/" + fileName); //$NON-NLS-1$
			URL url = FileLocator.find(getBundle(), path, null);
			if (url != null) {
				ImageDescriptor desc = ImageDescriptor.createFromURL(url);
				registry.put(key, desc);
			}
		} catch (Exception e) {
		}
	}

	/***************************************************************************
	 * FONT REGISTRY
	 **************************************************************************/
	private FontRegistry fontRegistry;

	private FontRegistry getFontRegistry() {
		if (fontRegistry == null) {
			fontRegistry = new FontRegistry(Display.getCurrent());

			fontRegistry.put(ITaxEditorConstants.DATASOURCE_FONT,
					new FontData[] { new FontData("Arial", 8, SWT.NONE) });
			fontRegistry.put(ITaxEditorConstants.MENU_ITEM_ITALICS_FONT,
					new FontData[] { new FontData("Arial", 9, SWT.ITALIC) });
			fontRegistry.put(ITaxEditorConstants.ACCEPTED_TAXON_FONT,
					new FontData[] { new FontData("Georgia", 12, SWT.NONE) });
			fontRegistry.put(ITaxEditorConstants.SYNONYM_FONT,
					new FontData[] { new FontData("Georgia", 10, SWT.NONE) });
			fontRegistry.put(ITaxEditorConstants.MISAPPLIEDNAME_FONT,
					new FontData[] { new FontData("Georgia", 10, SWT.NONE) });
			fontRegistry.put(ITaxEditorConstants.CONCEPT_FONT,
					new FontData[] { new FontData("Georgia", 10, SWT.NONE) });
			fontRegistry.put(ITaxEditorConstants.CHOOSE_NAME_TEXT_FONT,
					new FontData[] { new FontData("Arial", 12, SWT.BOLD) });
			fontRegistry.put(ITaxEditorConstants.DEFAULT_PROMPT_FONT,
					new FontData[] { new FontData("Georgia", 10, SWT.ITALIC) });			
		}
		return fontRegistry;
	}

	public Font getFont(String key) {
		return getFontRegistry().get(key);
	}

	/**
	 * @param font
	 */
	public Font italicizeFont(Font font) {
		FontData fontData = font.getFontData()[0];
		fontData.setStyle(fontData.getStyle() | SWT.ITALIC);
		Font italFont = new Font(Display.getDefault(), fontData);
		
		int i = 0;
		String fontName = "italic";
		Set keySet = fontRegistry.getKeySet();
		while (keySet.contains(fontName + i)) {
			i++;
		}
		fontRegistry.put(fontName + i, italFont.getFontData());
		
		return italFont;
	}

	/***************************************************************************
	 * COLOR MAP
	 **************************************************************************/
	private static HashMap<String, Color> colorRegistry;
	
	public Color getColor(String key) {
		return getColorRegistry().get(key);
	}
	
	private HashMap<String, Color> getColorRegistry() {
		if (colorRegistry == null) {
			 colorRegistry = new HashMap<String, Color>();
			 
			 colorRegistry.put(ITaxEditorConstants.GROUP_GRAY_BKG_COLOR,
					 new Color(null, 240, 240, 240));
			 colorRegistry.put(ITaxEditorConstants.PROP_SHEET_RED,
					 new Color(null, 255, 0, 0));			 
		}
		return colorRegistry;
	}
	
	private void disposeColors() {
		if (colorRegistry == null) {
			return;
		}
        for (Color color : colorRegistry.values()) {
        	color.dispose();
        }
        colorRegistry.clear();
	}

	/***************************************************************************
	 * PROPERTY SHEET
	 **************************************************************************/
	
	private Tree propertySheetTree;
	
	public void setPropertySheetTree(Tree tree) {
		this.propertySheetTree = tree;
	}
	
	public Tree getPropertySheetTree() {
		return propertySheetTree;
	}

	private PropertySheetPage propertySheetPage;
	
	public void setPropertySheetPage(PropertySheetPage page) {
		this.propertySheetPage = page;
	}
	
	public PropertySheetPage getPropertySheetPage() {
		return propertySheetPage;
	}
}